<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

namespace Vvveb\Plugins\Omnipay\Controller;

use Omnipay\Omnipay;
use function Vvveb\__;
use Vvveb\Controller\Base;
use function Vvveb\getSetting;
use Vvveb\System\Cart\Order;
use Vvveb\System\Sites;
use function Vvveb\url;

include __DIR__ . '/../../vendor/autoload.php';

class Index extends Base {
	protected $namespace = 'omnipay';

	protected $gateway;

	protected $order;

	protected $orderData;

	protected $settings;

	protected $successUrl;

	protected $cancelUrl;

	function ipn() {
		die('ipn');
	}

	private function errorMessage($message) {
		$this->response->setType('text');
		$this->response->output($message);

		die(0);
	}

	function init() {
		$gateway = $this->request->get['gateway'];
		$index   = $this->request->get['index'];
		$userKey = $this->request->get['key'];
		$orderId = $this->request->get['order_id'];
		$key     = \Vvveb\getConfig('app.key');

		if ($userKey != $key) {
			$this->errorMessage(__('Invalid key!'));
		}

		$this->order     = Order::getInstance();
		$this->orderData = $this->order->get($orderId)['order'] ?? [];

		if (! $this->orderData) {
			$this->errorMessage(__('Invalid order!'));
		}

		if (isset($this->orderData['payment_status_id']) && $this->orderData['payment_status_id'] > 1) {
			$this->errorMessage(__('Order payment status already set!'));
		}
		//var_dump($this->orderData);
		$this->settings = getSetting($this->namespace, null, []);
		$options        = $this->settings[$index] ?? false;

		if (! $options) {
			$this->errorMessage(__('Invalid gateway settings!'));
		}

		$this->gateway = Omnipay::create($gateway);
		$parameters    = [];

		foreach ($this->gateway->getDefaultParameters() as $key => $value) {
			if (isset($options[$key])) {
				$parameters[$key] = $options[$key];
			}
		}
		$this->gateway->initialize($parameters);
		$site = Sites :: getSiteData();

		$urlParams  = ['host' => $site['host'] ?? false, 'gateway' => $gateway, 'index' => $index, 'key' => $key, 'scheme' => $_SERVER['REQUEST_SCHEME'] ?? 'http'];
		$successUrl = url('plugins/omnipay/index/success', $urlParams);
		$cancelUrl  = url('plugins/omnipay/index/cancel', $urlParams);

		$this->successUrl = $successUrl;
		$this->cancelUrl  = $cancelUrl;
	}

	function success() {
		$orderId  = $this->request->get['order_id'];
		$order    = Order::getInstance();

		$parameters = [
			'amount'    => round(floatval($this->orderData['total']), 2),
			'currency'  => $this->orderData['currency'],
			'returnUrl' => $this->successUrl,
			'cancelUrl' => $this->cancelUrl,
		];

		$response = $this->gateway->completePurchase($parameters)->send();

		if ($response->isSuccessful()) {
			// Payment successful
			$transactionId = $response->getTransactionReference();
			$order->edit(['payment_status_id' => $this->settings['success_payment_status_id']], $orderId);

			return $this->redirect('checkout/confirm/index');
		} else {
			$order->edit(['payment_status_id' => $this->settings['error_payment_status_id']], $orderId);
			// Payment failed
			error_log($response->getMessage());
		}

		die(0);

		return false;
	}

	function cancel() {
		$orderId  = $this->request->get['order_id'];
		$order    = Order::getInstance();
		$order->edit(['payment_status_id' => $this->settings['error_payment_status_id']], $orderId);

		$transactionId = $response->getTransactionReference();
	}

	function index() {
	}
}
