<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

namespace Vvveb\Plugins\Omnipay;

use Omnipay\Omnipay;
use function Vvveb\isSecure;
use Vvveb\Sql\Region_groupSQL;
use Vvveb\System\PaymentMethod;
use Vvveb\System\Sites;
use function Vvveb\url;

include __DIR__ . '/vendor/autoload.php';

if (! defined('V_VERSION')) {
	die('Invalid request!');
}

class Payment extends PaymentMethod {
	private $namespace = 'omnipay';

	private $method_data = [
		'name'            => 'omnipay',
		'title'           => 'Omnipay',
		'description'     => 'Omnipay',
		'text'            => '',
		'region_id'       => 0,
		'cost'            => 0,
		'tax_type_id'     => 0,
		'region_group_id' => 0,
	];

	public function getMethod($checkoutInfo = [], $options = []) {
		$this->method_data         = $options + $this->method_data;
		$this->method_data['cost'] = floatval($this->method_data['cost']);
		//$this->getCost();

		//if set only for specific region(s) check if address matches region
		$region_group_id     = $this->method_data['region_group_id'];
		$shipping_country_id = $checkoutInfo['shipping_country_id'] ?? false;
		$shipping_region_id  = $checkoutInfo['shipping_region_id'] ?? 0;

		if ($region_group_id != 0) {
			$region  = new Region_groupSQL();
			$regions = [];
			$params  = ['region_group_id' => (int)$region_group_id, 'country_id' => (int)$shipping_country_id, 'region_id' => (int)$shipping_region_id];
			$regions = $region->isRegion($params);

			if (isset($regions['count']) && $regions['count'] == 0) {
				return [];
			}
		}

		$template = DIR_PUBLIC . "plugins/{$this->namespace}/{$this->method_data['gateway']}-form.html";

		if (APP == 'app' && file_exists($template)) {
			$form                             = file_get_contents($template);
			$this->method_data['render']      = $form;
		}

		return $this->method_data;
	}

	public function init() {
		//remove previous total and tax
		$this->cart->removeTotal($this->namespace);
		$this->cart->removeTax($this->namespace);
	}

	public function setMethod() {
		if ($this->method_data['cost'] || $this->method_data['text']) {
			if ($this->method_data['tax_type_id']) {
				$this->cart->addTax($this->namespace, $this->method_data['cost'], $this->method_data['tax_type_id']);
			}

			$this->cart->addTotal($this->namespace, $this->method_data['title'] , $this->method_data['cost'], $this->method_data['text']);
		}
	}

	public function authorize($checkoutInfo = []) {
		//$site = Sites :: getSiteData();

		$gateway = $this->method_data['gateway'];
		$index   = $this->method_data['index'];
		$key     = \Vvveb\getConfig('app.key');

		$urlParams  = ['host' => $checkoutInfo['site_url'] ?? false, 'gateway' => $gateway,  'key' => $key, 'index' => $index, 'order_id' => $checkoutInfo['order_id'], 'scheme' => isSecure() ? 'https' : 'http'];
		$successUrl = url('plugins/omnipay/index/success', $urlParams);
		$cancelUrl  = url('plugins/omnipay/index/cancel', $urlParams);

		$this->gateway = Omnipay::create($gateway);
		$parameters    = [];

		foreach ($this->gateway->getDefaultParameters() as $key =>  $value) {
			$parameters[$key] = $this->method_data[$key] ?? $value;
		}
		$this->gateway->initialize($parameters);

		//error_log(print_r($this->gateway->getDefaultParameters(), 1));
		error_log(print_r($parameters, 1));

		$currency = $checkoutInfo['currency'];
		//amount must be in cents so multiply with 100
		//$amount = intval(round(floatval($checkoutInfo['total']), 2) * 100);
		$amount = round(floatval($checkoutInfo['total']), 2);

		$options = [
			'amount'        => $amount,
			'currency'      => $currency,
			'returnUrl'     => $successUrl,
			'cancelUrl'     => $cancelUrl,
			'transactionId' => $checkoutInfo['order_id'],
		];

		if (isset($checkoutInfo['card'])) {
			$options['card'] = $checkoutInfo['card'];
		}

		error_log(print_r($options, 1));

		$response = $this->gateway->purchase($options)->send();

		if ($response->isRedirect()) {
			// Redirect to PayPal
			$response->redirect();
			error_log('redirect');

			return true;
		} else {
			if ($response->isSuccessful()) {
				error_log('success');

				return true;
			} else {
				// Payment failed
				$message = $response->getMessage();
				error_log('failed ' . $message);
			}

			return false;
		}
	}
}
